/*
** Filename: FT232Uart.c
**
** Automatically created by Application Wizard 1.2.4.1
**
** Part of solution Solution_1 in project FT232Uart
**
** Comments:
**
** Important: Sections between markers "FTDI:S*" and "FTDI:E*" will be overwritten by
** the Application Wizard
*/

#include "FT232Uart.h"

/* Default settings for UART interface */
#define DEF_UART_BAUD		UART_BAUD_9600
#define DEF_UART_DATA_BITS	UART_DATA_BITS_8
#define DEF_UART_FLOW		UART_FLOW_RTS_CTS
#define DEF_UART_STOP_BITS	UART_STOP_BITS_1
#define DEF_UART_PARITY		UART_PARITY_NONE

/* Default settings for FT232 interface */
#define DEF_FT232_BAUD		UART_BAUD_9600
#define DEF_FT232_DATA_BITS	UART_DATA_BITS_8
#define DEF_FT232_FLOW		UART_FLOW_NONE
#define DEF_FT232_STOP_BITS	UART_STOP_BITS_1
#define DEF_FT232_PARITY	UART_PARITY_NONE

/* FTDI:STP Thread Prototypes */
vos_tcb_t *tcbFT232UART;
vos_tcb_t *tcbUARTFT232;

void FT232UART();
void UARTFT232();
/* FTDI:ETP */

/* FTDI:SDH Driver Handles */
VOS_HANDLE hUSBHOST_1;                 // USB Host Port 1
VOS_HANDLE hUART;                      // UART Interface Driver
VOS_HANDLE hUSBHOST_FT232;             // Connects to FT232 devices using the USB Host Interface
/* FTDI:EDH */

vos_mutex_t mInit;

/* Declaration for IOMUx setup function */
void iomux_setup(void);

/* Main code - entry point to firmware */
void main(void)
{
	/* FTDI:SDD Driver Declarations */
	// UART Driver configuration context
	uart_context_t uartContext;
	// USB Host configuration context
	usbhost_context_t usbhostContext;

	/* FTDI:EDD */

	/* FTDI:SKI Kernel Initialisation */
	vos_init(50, VOS_TICK_INTERVAL, VOS_NUMBER_DEVICES);
	vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);
	vos_set_idle_thread_tcb_size(512);
	/* FTDI:EKI */

	iomux_setup();

	/* FTDI:SDI Driver Initialisation */
	// Initialise UART
	uartContext.buffer_size = VOS_BUFFER_SIZE_128_BYTES;
	uart_init(VOS_DEV_UART, &uartContext);

	// Initialise USB Host FT232 Driver
	usbHostFt232_init(VOS_DEV_USBHOST_FT232);

	// Initialise USB Host
	usbhostContext.if_count = 8;
	usbhostContext.ep_count = 16;
	usbhostContext.xfer_count = 2;
	usbhostContext.iso_xfer_count = 2;
	usbhost_init(VOS_DEV_USBHOST_1, -1, &usbhostContext);
	/* FTDI:EDI */

	// NOTE: Equal priority threads
	/* FTDI:SCT Thread Creation */
	tcbFT232UART = vos_create_thread_ex(20, 2048, FT232UART, "FT232UART", 0);
	tcbUARTFT232 = vos_create_thread_ex(20, 2048, UARTFT232, "UARTFT232", 0);
	/* FTDI:ECT */

	vos_init_mutex(&mInit, 1);

	vos_start_scheduler();

main_loop:
	goto main_loop;
}

/* FTDI:SSP Support Functions */

unsigned char usbhost_connect_state(VOS_HANDLE hUSB)
{
	unsigned char connectstate = PORT_STATE_DISCONNECTED;
	usbhost_ioctl_cb_t hc_iocb;

	if (hUSB)
	{
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_GET_CONNECT_STATE;
		hc_iocb.get = &connectstate;
		vos_dev_ioctl(hUSB, &hc_iocb);

		// repeat if connected to see if we move to enumerated
		if (connectstate == PORT_STATE_CONNECTED)
		{
			vos_dev_ioctl(hUSB, &hc_iocb);
		}
	}

	return connectstate;
}

VOS_HANDLE ft232_host_attach(VOS_HANDLE hUSB, unsigned char devHostFT232, unsigned char ftport)
{
	usbhost_device_handle_ex ifFT232;
	usbhost_ioctl_cb_t hc_iocb;
	usbhost_ioctl_cb_vid_pid_t hc_iocb_vendor;
	common_ioctl_cb_t ft232_iocb;
	usbhostft232_ioctl_cb_attach_t ft232_att;
	VOS_HANDLE hHostFT232;

	// find FT232 class device
	hc_iocb_vendor.vid = USB_VID_FTDI;
	hc_iocb_vendor.pid = USB_PID_FTDI_FT232;

	// user ioctl to find first FT232 device
	hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_DEVICE_FIND_HANDLE_BY_VID_PID;
	hc_iocb.handle.dif = NULL;
	hc_iocb.set = &hc_iocb_vendor;
	hc_iocb.get = &ifFT232;

	if (vos_dev_ioctl(hUSB, &hc_iocb) != USBHOST_OK)
	{
		return NULL;
	}

	// now we have a device, intialise the FT232 driver for it
	hHostFT232 = vos_dev_open(devHostFT232);

	// perform attach
	ft232_att.hc_handle = hUSB;
	ft232_att.ifDev = ifFT232;
	ft232_att.ftPort = ftport;

	ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_ATTACH;
	ft232_iocb.set = &ft232_att;
	ft232_iocb.get = NULL;

	if (vos_dev_ioctl(hHostFT232, &ft232_iocb) != USBHOSTFT232_OK)
	{
		vos_dev_close(hHostFT232);
		hHostFT232 = NULL;
	}

	return hHostFT232;
}

void ft232_host_detach(VOS_HANDLE hHostFT232)
{
	common_ioctl_cb_t ft232_iocb;

	if (hHostFT232)
	{
		ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_DETACH;

		vos_dev_ioctl(hHostFT232, &ft232_iocb);
		vos_dev_close(hHostFT232);
	}
}

/* FTDI:ESP */

void open_drivers(void)
{
	/* Code for opening and closing drivers - move to required places in Application Threads */
	/* FTDI:SDA Driver Open */
	hUSBHOST_1 = vos_dev_open(VOS_DEV_USBHOST_1);
	hUART = vos_dev_open(VOS_DEV_UART);
	/* FTDI:EDA */
}

void attach_drivers(void)
{
	/* FTDI:SUA Layered Driver Attach Function Calls */
	/* Attach to PORT A on multi-port devices */
	hUSBHOST_FT232 = ft232_host_attach(hUSBHOST_1, VOS_DEV_USBHOST_FT232, USBHOSTFT232_PORTA);
	/* FTDI:EUA */
}

void close_drivers(void)
{
	/* FTDI:SDB Driver Close */
	vos_dev_close(hUSBHOST_1);
	vos_dev_close(hUART);
	/* FTDI:EDB */
}

/* Application Threads */

void FT232UART()
{
	unsigned char status;

	// communications ioctl request blocks
	common_ioctl_cb_t ft232_iocb;
	common_ioctl_cb_t uart_iocb;

	// Host Controller ioctl request block
	usbhost_ioctl_cb_t hc_iocb;

	unsigned char leds;

	unsigned char buffer[64];
	unsigned short dataAvail, actual;

	open_drivers();

	/* UART ioctl call to enable DMA and link to DMA driver */
	uart_iocb.ioctl_code = VOS_IOCTL_COMMON_ENABLE_DMA;
	uart_iocb.set.param = DMA_ACQUIRE_AND_RETAIN;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set baud rate
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_BAUD_RATE;
	uart_iocb.set.uart_baud_rate = DEF_UART_BAUD;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set flow control
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_FLOW_CONTROL;
	uart_iocb.set.param = DEF_UART_FLOW;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set data bits
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_DATA_BITS;
	uart_iocb.set.param = DEF_UART_DATA_BITS;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set stop bits
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_STOP_BITS;
	uart_iocb.set.param = DEF_UART_STOP_BITS;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set parity
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_PARITY;
	uart_iocb.set.param = DEF_UART_PARITY;
	vos_dev_ioctl(hUART, &uart_iocb);

	// Host Controller Setup
	hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_SET_HANDLE_MODE_EXTENDED;
	vos_dev_ioctl(hUSBHOST_1, &hc_iocb);

	// GPIO Set LED pins to outputs
	vos_gpio_set_port_mode(GPIO_PORT_A, 0xff);  // set all as output

	do
	{
		// wait for enumeration to complete
		vos_delay_msecs(250);
		leds = LED3;
		vos_gpio_write_port(GPIO_PORT_A, leds);
		vos_delay_msecs(250);
		leds = 0;
		vos_gpio_write_port(GPIO_PORT_A, leds);

		hUSBHOST_FT232 = ft232_host_attach(hUSBHOST_1, VOS_DEV_USBHOST_FT232, 0);

		if (hUSBHOST_FT232)
		{
			leds = LED4;
			vos_gpio_write_port(GPIO_PORT_A, leds);

			// FT232 set baud rate
			uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_BAUD_RATE;
			uart_iocb.set.uart_baud_rate = DEF_FT232_BAUD;
			vos_dev_ioctl(hUSBHOST_FT232, &uart_iocb);

			// FT232 set flow control
			uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_FLOW_CONTROL;
			uart_iocb.set.param = DEF_FT232_FLOW;
			vos_dev_ioctl(hUSBHOST_FT232, &uart_iocb);

			// FT232 set data bits
			uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_DATA_BITS;
			uart_iocb.set.param = DEF_FT232_DATA_BITS;
			vos_dev_ioctl(hUSBHOST_FT232, &uart_iocb);

			// FT232 set stop bits
			uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_STOP_BITS;
			uart_iocb.set.param = DEF_FT232_STOP_BITS;
			vos_dev_ioctl(hUSBHOST_FT232, &uart_iocb);

			// FT232 set parity
			uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_PARITY;
			uart_iocb.set.param = DEF_FT232_PARITY;
			vos_dev_ioctl(hUSBHOST_FT232, &uart_iocb);

			// start polling to ensure we receive data in the RX Queue
			uart_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_START_POLL;
			vos_dev_ioctl(hUSBHOST_FT232, &uart_iocb);

			// Signal other thread to say everything is initialised and ready
			vos_unlock_mutex(&mInit);

			do
			{
				uart_iocb.ioctl_code = VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS;
				status = vos_dev_ioctl(hUSBHOST_FT232, &uart_iocb);

				if (status != USBHOSTFT232_OK)
				{
					leds = LED5;
					vos_gpio_write_port(GPIO_PORT_A, leds);
					break;
				}

				dataAvail = uart_iocb.get.queue_stat;

				if (dataAvail > sizeof(buffer))
				{
					dataAvail = sizeof(buffer);
				}

				if (dataAvail > 0)
				{
					// read from FT232 device
					status = vos_dev_read(hUSBHOST_FT232, buffer, dataAvail, &actual);

					if (status != USBHOSTFT232_OK)
					{
						leds = LED5;
						vos_gpio_write_port(GPIO_PORT_A, leds);
						break;
					}

					leds = 0;
					vos_gpio_write_port(GPIO_PORT_A, leds);

					// then write to UART
					status = vos_dev_write(hUART, buffer, actual, NULL);

					if (status != UART_OK)
					{
						leds = LED5;
						vos_gpio_write_port(GPIO_PORT_A, leds);
						break;
					}

					leds = LED4;
					vos_gpio_write_port(GPIO_PORT_A, leds);
				}
			}
			while (1);

			vos_lock_mutex(&mInit);

			ft232_host_detach(hUSBHOST_FT232);
		}
	}
	while (1);
}

void UARTFT232()
{
	unsigned char status;

	common_ioctl_cb_t uart_iocb;

	unsigned char leds;

	unsigned char buffer[64];
	unsigned short dataAvail, actual;

	do
	{
		// wait for other thread to complete initialisation sequence
		vos_lock_mutex(&mInit);

		uart_iocb.ioctl_code = VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS;
		vos_dev_ioctl(hUART, &uart_iocb);

		dataAvail = uart_iocb.get.queue_stat;

		if (dataAvail > sizeof(buffer))
		{
			dataAvail = sizeof(buffer);
		}

		if (dataAvail > 0)
		{
			// read from UART
			status = vos_dev_read(hUART, buffer, dataAvail, &actual);

			if (status != UART_OK)
			{
				leds = LED5;
				vos_gpio_write_port(GPIO_PORT_A, leds);
				break;
			}

			leds = 0;
			vos_gpio_write_port(GPIO_PORT_A, leds);

			// then write to FT232 device
			status = vos_dev_write(hUSBHOST_FT232, buffer, actual, NULL);

			if (status != USBHOSTFT232_OK)
			{
				leds = LED5;
				vos_gpio_write_port(GPIO_PORT_A, leds);
				break;
			}

			leds = LED4;
			vos_gpio_write_port(GPIO_PORT_A, leds);
		}

		vos_unlock_mutex(&mInit);
	}
	while (1);
}

